# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# <pep8 compliant>

bl_info = {
    "name": "Physics tools",
    "description": "Assorted tools to help controlling physics settings",
    "author": "Sergey Sharybin",
    "version": (0, 1),
    "blender": (2, 62, 3),
    "location": "Blender Game Engine > Physics buttons",
    "wiki_url": "",
    "tracker_url": "",
    "category": "Object"}

import bpy
from bpy.types import Operator, Panel
from mathutils import Vector


class OBJECT_OT_set_objects_mass(Operator):
    """Set mass for all selected objects"""

    bl_idname = "object.set_objects_mass"
    bl_label = "Set Mass"
    bl_space_type = 'VIEW_3D'

    def _getObjectBBMinMax(self, ob):
        min_co = Vector((1000000.0, 1000000.0, 1000000.0))
        max_co = -min_co
        for i in range(0, 8):
            bb_vec = ob.matrix_world * Vector(ob.bound_box[i])

            min_co[0] = min(bb_vec[0], min_co[0])
            min_co[1] = min(bb_vec[1], min_co[1])
            min_co[2] = min(bb_vec[2], min_co[2])
            max_co[0] = max(bb_vec[0], max_co[0])
            max_co[1] = max(bb_vec[1], max_co[1])
            max_co[2] = max(bb_vec[2], max_co[2])

        return (min_co, max_co)

    def _getObjectVolume(self, ob):
        min_co, max_co = self._getObjectBBMinMax(ob)

        x = max_co[0] - min_co[0]
        y = max_co[1] - min_co[1]
        z = max_co[2] - min_co[2]

        volume = x * y * z

        return volume

    def execute(self, context):
        scene = context.scene

        min_mass = scene.PhysicsMinMass
        max_mass = scene.PhysicsMaxMass

        min_volume = None
        max_volume = None

        for ob in context.selected_objects:
            volume = self._getObjectVolume(ob)

            if min_volume is None or volume < min_volume:
                min_volume = volume

            if max_volume is None or volume > max_volume:
                max_volume = volume

        for ob in context.selected_objects:
            volume = self._getObjectVolume(ob)

            if max_volume != min_volume:
                fac = (volume - min_volume) / (max_volume - min_volume)
            else:
                fac = 0.5

            mass = min_mass + fac * (max_mass - min_mass)

            ob.game.mass = mass

        return {'FINISHED'}


class VIEW3D_PT_modifiers(Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "physics"
    bl_label = "Mass"

    def draw(self, context):
        scene = context.scene
        layout = self.layout

        col = layout.column()
        row = col.row()
        row.prop(scene, "PhysicsMinMass", text="Min")
        row.prop(scene, "PhysicsMaxMass", text="Max")
        col.operator("object.set_objects_mass")


def register():
    bpy.types.Scene.PhysicsMinMass = bpy.props.FloatProperty(
            name="Min. Mass",
            default=1.0,
            description="Mas of smallest object")

    bpy.types.Scene.PhysicsMaxMass = bpy.props.FloatProperty(
            name="Max. Mass",
            default=10.0,
            description="Mas of biggest object")

    bpy.utils.register_module(__name__)


def unregister():
    del bpy.types.Scene.PhysicsMinMass
    del bpy.types.Scene.PhysicsMaxMass

    bpy.utils.unregister_module(__name__)


if __name__ == "__main__":
    register()
